@@ -69,14 +69,10 @@
 #include <gnutls/crypto.h>
 #endif
 
+#ifndef _XOPEN_SOURCE_EXTENDED
 #define _XOPEN_SOURCE_EXTENDED
-#ifdef HAVE_NCURSESW_NCURSES_H
-#include <ncursesw/ncurses.h>
-#elif HAVE_NCURSES_NCURSES_H
-#include <ncurses/ncurses.h>
-#else
-#include <ncurses.h>
 #endif
+#include <ncurses.h>
 
 
 // GnuTLS / libgcrypt functions
